@@ -50,9 +50,9 @@
 #endif
 
 /* needed to get localtime_r on Solaris */
-#ifdef sun
-#ifndef __EXTENSION__
-#define __EXTENSION__
+#ifdef __sun
+#ifndef __EXTENSIONS__
+#define __EXTENSIONS__
 #endif
 #endif
 
