@@ -289,7 +289,7 @@ xsltCompMatchAdd(xsltParserContextPtr ctxt, xsltCompMatchPtr comp,
 	    xsltAllocateExtra(ctxt->style);
     }
     if (op == XSLT_OP_PREDICATE) {
-	comp->steps[comp->nbStep].comp = xmlXPathCompile(value);
+	comp->steps[comp->nbStep].comp = xsltXPathCompile(ctxt->style, value);
     }
     comp->nbStep++;
     return (0);
@@ -1643,6 +1643,7 @@ xsltCompileLocationPathPattern(xsltParserContextPtr ctxt) {
 	 */
 	NEXT;
 	NEXT;
+	ctxt->comp->priority = 0.5;	/* '//' means not 0 priority */
 	xsltCompileRelativePathPattern(ctxt, NULL);
     } else if (CUR == '/') {
 	/*
@@ -1784,6 +1785,11 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 			 "xsltCompilePattern : parsing '%s'\n",
 			 element->pattern);
 #endif
+	/*
+	 Preset default priority to be zero.
+	 This may be changed by xsltCompileLocationPathPattern.
+	 */
+	element->priority = 0;
 	xsltCompileLocationPathPattern(ctxt);
 	if (ctxt->error) {
 	    xsltTransformError(NULL, style, node,
@@ -1801,42 +1807,36 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 	/*
 	 * Set-up the priority
 	 */
-	if (((element->steps[0].op == XSLT_OP_ELEM) ||
-	     (element->steps[0].op == XSLT_OP_ATTR)) &&
-	    (element->steps[0].value != NULL) &&
-	    (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = 0;
-#if 0
-	} else if ((element->steps[0].op == XSLT_OP_ROOT) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = 0;
-#endif
-	} else if ((element->steps[0].op == XSLT_OP_PI) &&
-		   (element->steps[0].value != NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = 0;
-	} else if ((element->steps[0].op == XSLT_OP_ATTR) &&
-		   (element->steps[0].value2 != NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.25;
-	} else if ((element->steps[0].op == XSLT_OP_NS) &&
-		   (element->steps[0].value != NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.25;
-	} else if ((element->steps[0].op == XSLT_OP_ATTR) &&
-		   (element->steps[0].value == NULL) &&
-		   (element->steps[0].value2 == NULL) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.5;
-	} else if (((element->steps[0].op == XSLT_OP_PI) ||
-		    (element->steps[0].op == XSLT_OP_TEXT) ||
-		    (element->steps[0].op == XSLT_OP_ALL) ||
-		    (element->steps[0].op == XSLT_OP_NODE) ||
-		    (element->steps[0].op == XSLT_OP_COMMENT)) &&
-		   (element->steps[1].op == XSLT_OP_END)) {
-	    element->priority = -0.5;
-	} else {
-	    element->priority = 0.5;
+	if (element->priority == 0) {	/* if not yet determined */
+	    if (((element->steps[0].op == XSLT_OP_ELEM) ||
+		 (element->steps[0].op == XSLT_OP_ATTR) ||
+		 (element->steps[0].op == XSLT_OP_PI)) &&
+		(element->steps[0].value != NULL) &&
+		(element->steps[1].op == XSLT_OP_END)) {
+		;	/* previously preset */
+	    } else if ((element->steps[0].op == XSLT_OP_ATTR) &&
+		       (element->steps[0].value2 != NULL) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.25;
+	    } else if ((element->steps[0].op == XSLT_OP_NS) &&
+		       (element->steps[0].value != NULL) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.25;
+	    } else if ((element->steps[0].op == XSLT_OP_ATTR) &&
+		       (element->steps[0].value == NULL) &&
+		       (element->steps[0].value2 == NULL) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.5;
+	    } else if (((element->steps[0].op == XSLT_OP_PI) ||
+		       (element->steps[0].op == XSLT_OP_TEXT) ||
+		       (element->steps[0].op == XSLT_OP_ALL) ||
+		       (element->steps[0].op == XSLT_OP_NODE) ||
+		       (element->steps[0].op == XSLT_OP_COMMENT)) &&
+		       (element->steps[1].op == XSLT_OP_END)) {
+			element->priority = -0.5;
+	    } else {
+		element->priority = 0.5;
+	    }
 	}
 #ifdef WITH_XSLT_DEBUG_PATTERN
 	xsltGenericDebug(xsltGenericDebugContext,
@@ -2057,6 +2057,7 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
     const xmlChar *name = NULL;
     xsltCompMatchPtr list = NULL;
     float priority;
+    int keyed = 0;
 
     if ((ctxt == NULL) || (node == NULL))
 	return(NULL);
@@ -2133,23 +2134,37 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		    list = curstyle->rootMatch;
 		else
 		    list = curstyle->elemMatch;
+		if (node->psvi != NULL) keyed = 1;
 		break;
-	    case XML_ATTRIBUTE_NODE:
+	    case XML_ATTRIBUTE_NODE: {
+	        xmlAttrPtr attr;
+
 		list = curstyle->attrMatch;
+		attr = (xmlAttrPtr) node;
+		if (attr->psvi != NULL) keyed = 1;
 		break;
+	    }
 	    case XML_PI_NODE:
 		list = curstyle->piMatch;
+		if (node->psvi != NULL) keyed = 1;
 		break;
 	    case XML_DOCUMENT_NODE:
-	    case XML_HTML_DOCUMENT_NODE:
+	    case XML_HTML_DOCUMENT_NODE: {
+	        xmlDocPtr doc;
+
 		list = curstyle->rootMatch;
+		doc = (xmlDocPtr) node;
+		if (doc->psvi != NULL) keyed = 1;
 		break;
+	    }
 	    case XML_TEXT_NODE:
 	    case XML_CDATA_SECTION_NODE:
 		list = curstyle->textMatch;
+		if (node->psvi != NULL) keyed = 1;
 		break;
 	    case XML_COMMENT_NODE:
 		list = curstyle->commentMatch;
+		if (node->psvi != NULL) keyed = 1;
 		break;
 	    case XML_ENTITY_REF_NODE:
 	    case XML_ENTITY_NODE:
@@ -2166,7 +2181,6 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		break;
 	    default:
 		break;
-
 	}
 	while ((list != NULL) &&
 	       ((ret == NULL)  || (list->priority > priority))) {
@@ -2210,7 +2224,7 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    }
 	}
 
-	if (node->_private != NULL) {
+	if (keyed) {
 	    list = curstyle->keyMatch;
 	    while ((list != NULL) &&
 		   ((ret == NULL)  || (list->priority > priority))) {
